
      PROGRAM  DAMPF

C   DAMPF : Ausgleichung einer gemessenene		      23.11.1997
C   -----   Dampfdruckkurve von Methylenchlorid 		   Weise
C							  DAMPF.FOR 3.03
C
C   nM, nt	 Anzahl der Modellgleichungen bzw. Konstanten
C   nx, ny	 Anzahl der Eingangs- bzw. Ergebnisgroessen
C		   ny = 3, nM = nt = nx = 10
C   t(nt)	 Konstanten, z.B. Stuetzstellen, Temperaturwerte
C		   in Grad C der Messung der Dampfdruckkurve
C   p(nx)	 Dampfdruckmesswerte in Torr
C   x(nx)	 Eingangsgroessen: Logarithmen von p, wobei p in Pa
C   y(ny)	 Ergebnisgroessen, Parameter der Ausgleichung
C		   (Anfangswerte fuer die Iteration gegeben)
C   Ux(nx,nx)	 Unsicherheitsmatrix zu x
C
C     Schalter fuer die nichtlineare Ausgleichung:
C   LUxd = .TRUE.  Ux ist diagonal
C   LUxE = .TRUE.  Ux liegt nicht vor. Es wird Ux = u0**2 * E
C		     gesetzt (E Einheitsmatrix), u0 wird berechnet
C   LMxy = .FALSE. Mx haengt nicht von y ab
C   LMxd = .TRUE.  Mx ist diagonal
C   LMxE = .TRUE.  Mx = E
C
C   Uy(ny,ny)	 Unsicherheitsmatrix zu y
C   u0		 globale Standardunsicherheit der Ausgleichung
C   t0, t1, t2	 Testwerte fuer Chiquadrat-, Korrelations- bzw.
C		   Vorzeichen-Korrelationskriterium, Soll: abs(ti) < 2,
C		   t0 nicht sinnvoll, wenn LUxE = .TRUE.
C   it		 Iterationsnummer

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER  ( Eins = 1.0D0, Hund = 1.0D2,
     -		   nM = 10, nt = nM, nx = nM, ny = 3 )
      LOGICAL	   LUxd, LUxE, LMxy, LMxd, LMxE
      EXTERNAL	   Antoine
      DOUBLE PRECISION
     -	t(nt), p(nx), x(nx), y(ny), Ux(nx,nx),
     -	M(nM), Mx(nM,nx), My(nM,ny),
     -	UxMxt(nx,nM), Kx(nM,nM), KxM(nM), KxMy(nM,ny),
     -	MytKxM(ny), Uy(ny,ny), z(nx)

C  -----  Eingangsdaten
      DATA
     -	LUxd, LUxE, LMxy, LMxd, LMxE
     -	  / .TRUE.,.TRUE.,.FALSE.,.TRUE.,.TRUE. /,
     -	t / -70.0,-52.1,-43.3,-33.4,-22.3,-15.7,-6.3,8.0,24.1,40.7 /,
     -	p / 1.0,5.0,10.0,20.0,40.0,60.0,100.0,200.0,400.0,760.0 /,
     -	y / 20.0,4000.0,273.0 /

C  -----  Eingangsdaten vorbereiten
C	  Druckwerte in Pa umrechnen und logarithmieren
      DO i = 1,nx
	x(i) = DLOG( p(i) * 101325.0D0/760.0D0 )
      END DO

C  =====  nichtlineare Ausgleichung
      CALL  NonLin ( nM,nt,nx,ny,t,x,y,Ux,
     -		     LUxd,LUxE,LMxy,LMxd,LMxE, Antoine,
     -		     Uy,z,u0,t0,t1,t2,it,
     -		     M,Mx,My,UxMxt,Kx,KxM,KxMy,MytKxM )

C  -----  Protokollausgabe
C   Mitteilung der Eingangsdaten, Ergebnisse und Unsicherheiten
      OPEN(6,FILE='DAMPF.AUS')
      WRITE(6,10)
 10   FORMAT(
     -	' DAMPF : Ausgleichung einer gemessenen'/
     -	' -----   Dampfdruckkurve von Methylenchlorid'//
     -	' Spalte 1 : Messwert-Nr. i'/
     -	' Spalte 2 : Temperaturwerte t(i) in Grad C'/
     -	' Spalte 3 : Dampfdruckmesswerte p(i) in Pa'/
     -	' Spalte 4 : ausgeglichene Dampfdruckwerte q(i) in Pa'/
     -	' Spalte 5 : x(i) = ln p(i)'/
     -	' Spalte 6 : z(i) = ln q(i)'/)
      DO i = 1,nx
	WRITE(6,11) i, t(i), DEXP(x(i)), DEXP(z(i)), x(i), z(i)
 11	FORMAT(1X,I2,F8.1,4(E14.5))
      END DO
      WRITE(6,12)
 12   FORMAT(/
     -	' Antoine-Gleichung als Ausgleichskurve :'/
     -	'     x = ln p = y(1) - y(2)/(t + y(3))'//
     -	' Spalte 1 : Parameter-Nr. k'/
     -	' Spalte 2 : Parameter y(k) als Zahl (k = 1)',
     -	' oder in Grad C (k = 2,3)'/
     -	' Spalte 3 : Standardunsicherheiten der Parameter',
     -	' in deren Einheiten'/
     -	' uebrige Spalten : Korrelationsmatrix der Parameter'/)
      DO k = 1,ny
	WRITE(6,13) k, y(k), DSQRT(Uy(k,k)),
     -	  (Uy(k,j)/DSQRT(Uy(k,k)*Uy(j,j)), j = 1,ny)
 13	FORMAT(1X,I2,2(E14.5),3(F8.3))
      END DO
      WRITE(6,14) u0, Hund*(DEXP(u0) - Eins), t0,t1,t2,it
 14   FORMAT(/
     -	' globale Standardunsicherheit u0 zu x  : ',E14.5/
     -	' globale relative Standardunsicherheit'/
     -	'    des ausgeglichenen Dampfdrucks q   : ',F8.3,' %'/
     -	' Chiquadrat-Testwert t0                : ',E14.5/
     -	' Korrelationstestwerte t1 und t2       : ',2(E14.5)/
     -	'    Soll: abs(ti) < 2 '/
     -	' Anzahl der Iterationen                : ',I4)
      CLOSE(6)
      END
C --------------

      SUBROUTINE  Antoine ( it,nM,nt,nx,ny, t,x,y, M,Mx,My )

C   Antoine : Berechnung der Modellfunktion M(t; x, y) und der
C   -------   Matrizen Mx und My ihrer partiellen Ableitungen
C	      (x Eingangsgroessen, y Ergebnisgroessen, t Konstanten)
C	hier: Antoine-Gleichung als Dampfdruckkurve
C	      M(t; x, y) = x - ( y(1) - y(2)/(t + y(3)) )
C	      x = ln p	(p Dampfdruck, t Temperatur, y Parameter)
C	    Eingabe:  it, nM, nt, nx, ny, t, x, y
C	    Ausgabe:  M, Mx, My
C
C   nM = nt = nx Anzahl der Modellgleichungen bzw. Konstanten
C   nx, ny = 3	 Anzahl der Eingangs- bzw. Ergebnisgroessen, nx >= ny
C   t(nt)	 Konstanten, hier Stuetzstellen, Temperatur
C   x(nx)	 Eingangsgroessen, x = ln p  (p Dampfdruck)
C   y(ny)	 Ergebnisgroessen, Parameter der Ausgleichung
C   M(nM)	 Werte des Modells M(t; x, y)
C   Mx(nM,nx) = ( d M(i)/d x(k) )  Ableitungen von M nach x
C   My(nM,ny) = ( d M(i)/d y(k) )  Ableitungen von M nach y
C   it		 Iterationsnummer. Nur wenn it = 1, Mx = E setzen

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER  ( Zero = 0.0D0, Eins = 1.0D0 )
      DOUBLE PRECISION	t(nt), x(nx), y(ny),
     -			M(nM), Mx(nM,nx), My(nM,ny)

      IF (nM.NE.nx .OR. nt.NE.nx .OR. ny.NE.3 .OR. nx.LT.ny) STOP
     -	  ' Antoine: Matrixdimensionen falsch. Fehlerstop'

C  -----  Mx = E ( nur wenn it = 1 )
      IF (it .EQ. 1) THEN
	DO i = 1,nM
	  DO k = 1,nx
	    Mx(i,k) = Zero
	  END DO
	  Mx(i,i) = Eins
	END DO
      ENDIF

C  -----  M und My
      DO i = 1,nM
	h = t(i) + y(3)
	M(i) = x(i) - y(1) + y(2) / h
	My(i,1) = -Eins
	My(i,2) =  Eins / h
	My(i,3) = -y(2) / h / h
      END DO

      RETURN
      END
C --------------

      SUBROUTINE  NonLin ( nM,nt,nx,ny,t,x,y,Ux,
     -			   LUxd,LUxE,LMxy,LMxd,LMxE, MODELL,
     -			   Uy,z,u0,t0,t1,t2,it,
     -			   M,Mx,My,UxMxt,Kx,KxM,KxMy,MytKxM )

C   NonLin : Nichtlineare Ausgleichung mittels der
C   ------   Modellgleichung M(t; x, y) = 0
C	     (x Eingangsgroessen, y Ergebnisgroessen, t Konstanten)
C	   Eingabe:  nM, nt, nx, ny, t, x, y, Ux,
C		     LUxd, LUxE, LMxy, LMxd, LMxE, MODELL
C	   Ausgabe:  y, Uy, z, u0, t0, t1, t2, it
C
C   nM, nt	 Anzahl der Modellgleichungen bzw. Konstanten
C   nx, ny	 Anzahl der Eingangs- bzw. Ergebnisgroessen
C		   Bedingung: ny <= nM <= nx
C   t(nt)	 Konstanten, z.B. Stuetzstellen
C   x(nx)	 Eingangsgroessen
C   y(ny)	 Ergebnisgroessen, Parameter der Ausgleichung
C   Ux(nx,nx)	 Unsicherheitsmatrix zu x
C
C     Schalter zur Beschleunigung der Ausgleichung in Sonderfaellen:
C   LUxd = .TRUE.  wenn Ux diagonal ist
C   LUxE = .TRUE.  wenn Ux = u0**2 * E zu setzen ist
C		     (bewirkt LUxd = .TRUE. und Ux = E. u0 wird
C		     berechnet, E Einheitsmatrix)
C   LMxy = .TRUE.  wenn Mx von y abhaengt
C   LMxd = .TRUE.  wenn Mx diagonal ist (erfordert nM = nx)
C   LMxE = .TRUE.  wenn Mx = E ist (bewirkt LMxy = .FALSE. und
C		     LMxd = .TRUE.)
C
C   MODELL	 Unterprogramm, das folgendes liefert:
C   M(nM)	 Werte des Modells M(t; x, y)
C   Mx(nM,nx)	 = ( d M(i)/d x(k) )  Ableitungen von M nach x
C   My(nM,ny)	 = ( d M(i)/d y(k) )  Ableitungen von M nach y
C
C     Verfahren (angehaengtes t und i bedeuten
C		 Transponierung bzw. Inversion einer Matrix):
C   UxMxt(nx,nM) = Ux * (Mx)t
C   Kx(nM,nM)	 = ( Mx * UxMxt )i
C   KxM(nM)	 = Kx * M
C   KxMy(nM,ny)  = Kx * My
C   MytKxM(ny)	 = (KxMy)t * M
C   Uy(ny,ny)	 = ( (My)t * KxMy )i	   Unsicherheitsmatrix zu y
C   y1(ny)	 = y - beta * Uy * MytKxM  verbessertes y ersetzt y
C   z(nx)	 = x - UxMxt * KxM	   ausgeglichene Werte zu x
C   chiq	 = (M)t * KxM
C
C   it		 Anzahl der Iterationen
C   u0		 globale Standardunsicherheit der Ausgleichung
C   t0		 Testwert fuer Iterationsabbruch und
C		   Chiquadrat-Kriterium
C   t1, t2	 Korrelationstestwerte,  Soll: abs(ti) < 2
C   eps 	 Genauigkeit von t0 fuer Iterationsabbruch
C   beta	 Konvergenzparameter. Verkleinern, wenn
C		   Iteration nicht konvergiert

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER  ( Zero = 0.0D0, Eins = 1.0D0,
     -		   itmax = 100,  beta = Eins, eps = 1.0D-5 )
      LOGICAL	   LUxd, LUxE, LMxy, LMxd, LMxE, LKxd
      DOUBLE PRECISION
     -	  t(nt), x(nx), y(ny), Ux(nx,nx),
     -	  M(nM), Mx(nM,nx), My(nM,ny),
     -	  UxMxt(nx,nM), Kx(nM,nM), KxM(nM),
     -	  KxMy(nM,ny), MytKxM(ny), Uy(ny,ny), z(nx)

C  -----  Ux = E setzen, wenn Ux = u0**2 * E,
C	  sowie Schalter setzen
      IF ( LUxE ) THEN
	LUxd = .TRUE.
	DO i = 1,nx
	  DO k = 1,nx
	    Ux(i,k) = Zero
	  END DO
	  Ux(i,i) = Eins
	END DO
      ENDIF
      IF ( LMxE ) LMxy = .FALSE.
      IF ( LMxE ) LMxd = .TRUE.
      LKxd = LUxd .AND. LMxd
      tt = Zero

C  -----  Matrixdimensionen pruefen
      IF ( (LMxd .AND. nM.NE.nx) .OR.
     -	   (.NOT.(ny.LE.nM .AND. nM.LE.nx)) ) STOP
     -	   'NonLin: Matrixdimensionen falsch. Fehlerstop 1'

C  =====  Iterationsschleife
      DO it = 1,itmax
	CALL MODELL ( it,nM,nt,nx,ny, t,x,y, M,Mx,My )

C  *****  Vorbereitung. Nur anfangs oder wenn Mx von y abhaengt
      IF (it.EQ.1 .OR. LMxy) THEN

      IF ( LKxd ) THEN
C  -----  Falls sowohl Ux als auch Mx diagonal sind, ist auch
C	  Kx = ( Mx * Ux * (Mx)t )i oder Kx = (Ux)i diagonal
	DO i = 1,nM
	  DO k = 1,nM
	    Kx(i,k) = Zero
	  END DO
	  h = Ux(i,i)
	  IF ( .NOT.LMxE ) h = h * Mx(i,i)**2
	  Kx(i,i) = Eins / h
	END DO
      GOTO 88
      ENDIF

      IF ( LMxd ) THEN
C  -----  Ux ist nichtdiagonal, Mx diagonal
C	  (Kx)i = Mx * Ux * (Mx)t  oder  (Kx)i = Ux
	DO i = 1,nM
	DO k = 1,i
	  h = Ux(i,k)
	  IF ( .NOT.LMxE ) h = Mx(i,i) * h * Mx(k,k)
	  Kx(i,k) = h
	  Kx(k,i) = h
	END DO
	END DO

       ELSE
C  -----  Mx ist nichtdiagonal
C	  UxMxt = Ux * (Mx)t
	DO i = 1,nx
	DO k = 1,nM
	  IF ( LUxd ) THEN
	    h = Ux(i,i) * Mx(k,i)
	   ELSE
	    h = Zero
	    DO j = 1,nx
	      h = h + Ux(i,j) * Mx(k,j)
	    END DO
	  ENDIF
	  UxMxt(i,k) = h
	END DO
	END DO

C  -----  (Kx)i = Mx * UxMxt
	DO i = 1,nM
	DO k = 1,i
	  h = Zero
	  DO j = 1,nx
	    h = h + Mx(i,j) * UxMxt(j,k)
	  END DO
	  Kx(i,k) = h
	  Kx(k,i) = h
	END DO
	END DO

      ENDIF

C  -----  Invertieren von (Kx)i (in Kx) ergibt Kx
	CALL DefInv ( nM,Kx,.TRUE.,Ld )
	IF (Ld .LT. nM) STOP
     -	   'NonLin: Matrix (Kx)i nicht invertierbar. Fehlerstop 2'

 88   CONTINUE
      ENDIF
C  *****  Ende der Vorbereitung

C  -----  KxM = Kx * M	und  chiq = (M)t * KxM
      chiq = Zero
      DO i = 1,nM
	IF ( LKxd ) THEN
	  h = Kx(i,i) * M(i)
	 ELSE
	  h = Zero
	  DO k = 1,nM
	    h = h + Kx(i,k) * M(k)
	  END DO
	ENDIF
	KxM(i) = h
	chiq = chiq + M(i) * h
      END DO

C  -----  KxMy = Kx * My
      DO i = 1,nM
      DO k = 1,ny
	IF ( LKxd ) THEN
	  h = Kx(i,i) * My(i,k)
	 ELSE
	  h = Zero
	  DO j = 1,nM
	    h = h + Kx(i,j) * My(j,k)
	  END DO
	ENDIF
	KxMy(i,k) = h
      END DO
      END DO

C  -----  MytKxM = (KxMy)t * M
      DO i = 1,ny
	h = Zero
	DO j = 1,nM
	  h = h + KxMy(j,i) * M(j)
	END DO
	MytKxM(i) = h
      END DO

C  -----  (Uy)i = (My)t * KxMy
      DO i = 1,ny
      DO k = 1,i
	h = Zero
	DO j = 1,nM
	  h = h + My(j,i) * KxMy(j,k)
	END DO
	Uy(i,k) = h
	Uy(k,i) = h
      END DO
      END DO

C  -----  Invertieren von (Uy)i (in Uy) ergibt Uy
      CALL DefInv ( ny,Uy,.TRUE.,Ld )
      IF (Ld .LT. ny) STOP
     -	 'NonLin: Matrix (Uy)i nicht invertierbar. Fehlerstop 3'

C  -----  Ausgabe des Testwerts t0 zur Iterationskontrolle
C	  und Abbruchbedingung pruefen
      t0 = (chiq - nM + ny) / DSQRT( 2.0D0 * (nM - ny) )
      WRITE(*,11) it,t0
 11   FORMAT(I3,D20.5)
      IF (DABS(t0 - tt).LT.eps .AND. it.GT.3) GOTO 99
      tt = t0

C  -----  y1 = y - beta * Uy * MytKxM
      DO i = 1,ny
	h = Zero
	DO k = 1,ny
	  h = h + Uy(i,k) * MytKxM(k)
	END DO
	y(i) = y(i) - beta * h
      END DO

      END DO
      it = itmax
 99   CONTINUE
C  =====  Ende der Iterationsschleife

C  -----  z = x - UxMxt * KxM
      DO i = 1,nx
	IF ( LMxd ) THEN
	  h = M(i) / Mx(i,i)
	 ELSE
	  h = Zero
	  DO k = 1,nM
	    h = h + UxMxt(i,k) * KxM(k)
	  END DO
	ENDIF
	z(i) = x(i) - h
      END DO

C  -----  u0, t1 und t2
      h0 = Zero
      h1 = Zero
      n2 = 0
      DO i = 1,nx
	h  = x(i) - z(i)
	h0 = h0 + h * x(i)
	IF (i .GT. 1) THEN
	  h  = h * (x(i-1) - z(i-1))
	  h1 = h1 + h
	  IF (h .GT. Zero) n2 = n2 + 1
	  IF (h .LT. Zero) n2 = n2 - 1
	ENDIF
      END DO
      u2 = h0 / (nx - ny)
      u0 = DSQRT( u2 )
      t1 = h1 / h0 * DSQRT( Eins * (nx - ny) )
      t2 = n2 / DSQRT( Eins * (nx - 1) )

C  -----  Uy = u0**2 * Uy, wenn Ux = u0**2 * E
      IF ( LUxE ) THEN
	DO i = 1,ny
	DO k = 1,ny
	  Uy(i,k) = u2 * Uy(i,k)
	END DO
	END DO
      ENDIF

      RETURN
      END
C --------------

      SUBROUTINE  DefInv ( n,S,Inv,d )

C   DefInv : Pruefen, ob eine symmetrische Matrix S positiv
C   ------   (semi)definit ist, und die Matrix S invertieren,
C	     wenn verlangt und wenn sie positiv definit ist
C							Weise 12.12.1993
C   Eingabe: n, S(n,n), Inv
C   Ausgabe:
C     Wenn S positiv (semi)definit ist: d = Rang(S) <= n, und
C	die obere Dreieckmatrix T mit (T)t*T = S. (T)t ersetzt S
C	(Cholesky-Zerlegung, (T)t transponierte Matrix T).
C     Wenn S positiv definit ist und Inv = .TRUE.: d = n,
C	und S wird durch ihre inverse Matrix (S)i ersetzt.
C     Sonst d = -1, und S ist ueberschrieben.

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER  ( Zero = 0.0D0, Eins = 1.0D0 )
      INTEGER	 d, h
      LOGICAL	 Inv
      DIMENSION  S(n,n)

      d = -1
      h =  0
      DO i = 1,n
      DO k = i,n
	p = S(i,k)
	IF (i .GT. 1) THEN
	  DO j = 1,i-1
	    p = p - S(i,j) * S(k,j)
	  END DO
	ENDIF
	IF (k .EQ. i) THEN
	  IF (p .LT. Zero) RETURN
	  IF (p .GT. Zero) THEN
	    S(i,i) = DSQRT( p )
	    h = h + 1
	  ENDIF
	 ELSE
	  IF (S(i,i) .NE. Zero) THEN
	    S(k,i) = p / S(i,i)
	   ELSE
	    IF (p .NE. Zero) RETURN
	  ENDIF
	ENDIF
      END DO
      END DO
      d = h
      IF (h.LT.n .OR. .NOT.Inv) RETURN

C   Invertieren der oberen Dreieckmatrix T ( (T)t liegt in S )
      DO k = 1,n
	S(k,k) = Eins / S(k,k)
	IF (k .NE. 1) THEN
	  DO i = k-1,1,-1
	    p = Zero
	    DO j = i+1,k
	      p = p + S(j,i) * S(j,k)
	    END DO
	    S(i,k) = -p * S(i,i)
	  END DO
	ENDIF
      END DO

C   (S)i = (T)i * ((T)i)t  ( (T)i liegt in S )
      DO i = n,1,-1
      DO k = 1,n
	IF (k .GT. i) THEN
	  S(i,k) = S(k,i)
	 ELSE
	  p = Zero
	  DO j = i,n
	    p = p + S(i,j) * S(k,j)
	  END DO
	  S(i,k) = p
	ENDIF
      END DO
      END DO

      RETURN
      END
C ==============
