
      PROGRAM  NSPEKT

C   NSPEKT : Fourier-Ausgleichung einer gemessenen	      23.11.1997
C   ------   Energieverteilung von Neutronen aus		   Weise
C	     einer 241Am-Be(alpha,n)-Neutronenquelle	 NSPEKT.FOR 3.03
C
C   nx		 Anzahl der Eingangsgroessen und der Stuetzstellen
C   ny > 2	 effektive Anzahl der Koeffizienten der Fourier-
C		   Ausgleichung (wird iterativ ermittelt), ny <= nx
C   En0, dEn	 Anfangswert der Neutronenenergie bzw. Energie-
C		   differenz der aequidistanten Stuetzstellen
C   En(nx)	 Stuetzstellen der Neutronenenergie
C   x(nx)	 Eingangsgroessen (als unkorreliert angenommen),
C		   Messergebnisse der Energieverteilung
C   z(nx)	 ausgeglichene Werte zu x
C   y(ny)	 Ergebnisgroessen, Fourier-Koeffizienten
C   Ux(nx)	 diagonale Unsicherheitsmatrix zu x
C   Uy(ny,ny)	 Unsicherheitsmatrix zu y
C   Uyi(nyi)	 inverse Matrix Uy, oberes Dreieck spaltenweise
C		   linear gespeichert, nyi = ny*(ny+1)/2
C   w(ny)	 Hilfsfeld und Standardunsicherheiten von Uy
C   u0		 globale Standardunsicherheit
C   A(nx,ny)	 Modellmatrix
C   Lrand = .FALSE.  A ist neu zu berechnen
C	  = .TRUE.   nur Spalte ny von A auffuellen
C   LUxE  = .FALSE.  Ux liegt vor
C	  = .TRUE.   Ux liegt nicht vor. Es wird Ux = u0**2 * E
C		     angenommen (E Einheitsmatrix), u0 wird berechnet
C   tt		 Parameter fuer den Iterationsabbruch, 0 <= tt <= 2
C   Krit = 0,1,2 = i   waehlt Testwert ti fuer den Iterationsabbruch
C		   durch ti < tt aus. Krit = 0 nicht sinnvoll, wenn
C		   LUxE = .TRUE.
C   t0, t1, t2	 Testwerte fuer Chiquadrat-, Korrelations- bzw.
C		   Vorzeichen-Korrelationskriterium, Soll: abs(ti) < 2

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER  ( Zero = 0.0D0,
     -		   nxmax = 250, nymax = 100, tt = Zero, Krit = 0,
     -		   nyimax = nymax*(nymax+1)/2 )
      LOGICAL	   Lrand, LUxE
      CHARACTER*80 TX
      DIMENSION    x(nxmax), En(nxmax), z(nxmax),
     -		   y(nymax), A(nxmax,nymax), w(nymax),
     -		   Ux(nxmax), Uy(nymax,nymax), Uyi(nyimax)

C  -----  Eingangsdaten einlesen und vorbereiten
      OPEN(5,FILE = 'WEI22005.TEK')
      READ(5,11) TX,TX
 11   FORMAT(A80/A80)
      READ(5,*) nx,nymin,En0,dEn
      DO i = 1,nx
	READ(5,*) x(i),Ux(i)
	En(i) = En0 + dEn * (i-1)
      END DO
      CLOSE(5)
      LUxE = .FALSE.

C  =====  Iterative Ausgleichung mit wachsendem ny
      Lrand = .FALSE.
      DO ny = nymin,nymax
	nyi = ny * (ny+1) / 2

C  -----  Modellmatrix A bilden oder ergaenzen
	CALL  Four ( nx,ny,En,Lrand, A )

C  -----  Lineare Ausgleichung
	CALL  Linear ( nx,ny,nyi,x,Ux,A,Lrand,LUxE,
     -		       z,y,Uy,Uyi,w,u0,t0,t1,t2 )
	Lrand = .TRUE.

C  -----  Testwerte zur Iterationskontrolle ausgeben
C	  und auf Abbruch pruefen
	WRITE(*,13) ny,t0,t1,t2
	tkrit = t0
	IF (Krit .EQ. 1) tkrit = t1
	IF (Krit .EQ. 2) tkrit = t2
	IF (tkrit .LT. tt) GOTO 99

      END DO
      ny = nymax
 99   CONTINUE
C  =====  Ende der Iteration

C  -----  Uy vervollstaendigen
      CALL Ukorr ( ny,Uy,w,u0,LUxE )

C  -----  Protokoll ausgeben
C	  Mitteilung der Eingangsdaten und Ergebnisse
      OPEN(6,FILE = 'NSPEKT.AUS')
      WRITE(6,12) ny
 12   FORMAT(
     -	' NSPEKT : Verteilung der Energie von Neutronen aus'/
     -	' ------   einer 241Am-Be(alpha,n)-Neutronenquelle'//
     -	' Fourier-Ausgleichung mit',I3,' Termen'//
     -	' Spalte 1 : Messwert-Nr. i'/
     -	' Spalte 2 : Neutronenenergie En(i) in MeV'/
     -	' Spalte 3 : Dichte x(i) der Emissionswahrscheinlichkeit',
     -	' in 1/MeV'/
     -	' Spalte 4 : ausgeglichene Dichte z(i) in 1/MeV'/
     -	' Spalte 5 : Residuum z(i) - x(i) in 1/MeV'/
     -	' Spalte 6 : Standardunsicherheit zu x(i) in 1/MeV'/)
      DO i = 1,nx
	h = u0
	IF ( .NOT.LUxE ) h = DSQRT( Ux(i) )
	WRITE(6,13) i,En(i),x(i),z(i),z(i)-x(i),h
 13	FORMAT(1X,I3,1X,6(E13.5))
      END DO

      WRITE(6,14)
 14   FORMAT(/
     -	' Spalte 1 : Fourier-Koeffizient-Nr. i'/
     -	' Spalte 2 : Fourierkoeffizient y(i) in 1/MeV'/
     -	' Spalte 3 : Standardunsicherheit zu y(i) in 1/MeV'/)
      DO i = 1,ny
	WRITE(6,13) i,y(i),w(i)
      END DO
C	Auf die Ausgabe der Korrelationskoeffizienten ist verzichtet

      WRITE(6,15) u0,t0,t1,t2,Krit,tt
 15   FORMAT(/
     -	' globale Standardunsicherheit u0 =',E13.5,'  1/MeV'/
     -	' Testwerte t0, t1, t2            =',3(E13.5)/
     -	' Soll: abs(ti) < 2'/
     -	' Iterationsabbruch durch t',I1,' < tt =',E13.5)
      CLOSE(6)

      END
C --------------

      SUBROUTINE  Ukorr ( n,U,w,u0,L )

C   Ukorr : Die Unsicherheitsmatrix U wird vervollstaendigt.
C   -----   U wird mit u0**2 multipliziert, wenn L = .TRUE.,
C	    das untere Dreieck wird mit den Korrelations-
C	    koefizienten besetzt, das Feld w mit den Standard-
C	    unsicherheiten

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      LOGICAL	 L
      DIMENSION  U(n,n), w(n)

      h = 1.0D0
      IF ( L ) h = u0
      DO i = 1,n
	w(i) = h * DSQRT( U(i,i) )
      END DO
      DO i = 1,n-1
      DO k = i+1,n
	U(i,k) = h * h * U(i,k)
	U(k,i) = U(i,k) / w(i) / w(k)
      END DO
      END DO

      RETURN
      END
C --------------

      SUBROUTINE  Four ( nx,ny,t,Lrand, A )

C   Four : Berechnen der Modellmatrix A der Modellgleichung
C   ----   x = A * y fuer eine Fourier-Ausgleichung
C	   (x Eingangsgroessen, y Ergebnisgroessen)
C	 Eingabe:  nx, ny, t, Lrand ;	Ausgabe:  A
C   Modell der Ausgleichung:
C
C   x(i) = y(1) + y(2) * cos(phi) +
C		+ Summe(k = 3,..,ny) y(k) * sin( (k-2)*phi )
C	   ( phi = pi * (t(i)-t(1))/(t(nx)-t(1)) ; i = 1,..,nx )
C
C   nx, ny     Anzahl der Eingangs- bzw. Ergebnisgroessen,
C		 nx >= ny >= 3
C   t(nx)      Stuetzstellen (moeglichst aequidistant)
C   A(nx,ny)   Modellmatrix
C   Lrand = .FALSE.  A wird neu berechnet
C	  = .TRUE.   nur Spalte ny von A wird aufgefuellt

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER  ( Eins = 1.0D0, pi = 3.141592653589793D0 )
      LOGICAL	   Lrand
      DIMENSION    t(nx), A(nx,ny)

      IF (nx.LT.ny .OR. ny.LT.3) STOP
     -	   'Four: Matrixdimensionen falsch. Fehlerstop'

      h = pi / (t(nx) - t(1))
      DO i = 1,nx
	phi = h * (t(i) - t(1))
	IF ( Lrand ) THEN
	  A(i,ny) = DSIN( (ny-2) * phi )
	 ELSE
	  A(i,1) = Eins
	  A(i,2) = DCOS( phi )
	  DO k = 3,ny
	    A(i,k) = DSIN( (k-2) * phi )
	  END DO
	ENDIF
      END DO

      RETURN
      END
C --------------

      SUBROUTINE  Linear ( nx,ny,nyi,x,Ux,A,Lrand,LUxE,
     -			   z,y,Uy,Uyi,w,u0,t0,t1,t2 )

C   Linear : Lineare Ausgleichung mittels der Modellgleichung
C   ------   x = A * y	bei diagonaler Unsicherheitsmatrix Ux zu x
C	     (x Eingangsgroessen, y Ergebnisgroessen, A Modellmatrix)
C	   Eingabe:  nx, ny, nyi, x, Ux, A, Lrand, LUxE
C	   Ausgabe:  z, y, Uy, w, u0, t0, t1, t2
C
C   nx, ny	 Anzahl der Eingangs- bzw. Ergebnisgroessen, nx >= ny
C   x(nx)	 Eingangsgroessen
C   z(nx)	 ausgeglichene Werte zu x
C   y(ny)	 Ergebnisgroessen, Parameter der Ausgleichung
C   Ux(nx)	 diagonale Unsicherheitsmatrix zu x. Es wird Ux = E
C		   gesetzt, wenn LUxE = .TRUE. (E Einheitsmatrix)
C   Uy(ny,ny)	 Unsicherheitsmatrix zu y, jedoch
C		   dividiert durch u0**2, wenn LUxE = .TRUE.
C   Uyi(nyi)	 inverse Matrix Uy, oberes Dreieck spaltenweise
C		   linear zwischengespeichert, nyi = ny*(ny+1)/2
C   w(ny)	 Hilfsfeld
C   u0		 globale Standardunsicherheit
C   A(nx,ny)	 Modellmatrix
C   Lrand = .FALSE.  A wurde neu berechnet
C	  = .TRUE.   nur Spalte ny wurde bei A hinzugefuegt
C   LUxE  = .FALSE.  Ux ist gegeben
C	  = .TRUE.   Ux ist nicht gegeben. Es wird Ux = u0**2 * E
C		     angenommen, u0 wird berechnet
C   t0, t1, t2	 Testwerte fuer Chiquadrat-, Korrelations- bzw.
C		   Vorzeichen-Korrelationskriterium, Soll: abs(ti) < 2
C
C   Verfahren (angehaengtes t und i bedeuten
C	       Transponierung bzw. Inversion einer Matrix):
C     Gegeben sind     x, Ux, A
C     Berechnet werden u.a.
C	Uyi = (A)t * (Ux)i * A ;   w = (A)t * (Ux)i * x ;
C	Uy  = (Uyi)i ;	 y = Uy * w ;	z = A * y ;
C	chi**2 = (z - x)t * (Ux)i * (z - x)

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER  ( Zero = 0.0D0, Eins = 1.0D0 )
      LOGICAL	   Lrand, LUxE
      DIMENSION    x(nx), z(nx), y(ny), A(nx,ny), w(ny),
     -		   Ux(nx), Uy(ny,ny), Uyi(nyi)

      IF (nx.LT.ny .OR. nyi.NE. ny*(ny+1)/2) STOP
     -	 'Linear: Matrixdimensionen falsch. Fehlerstop 1'

C  -----  Wenn Lrand = .TRUE., Matrizen w und Uxi nur ergaenzen
      kmin = 1
      IF ( Lrand ) kmin = ny
      l = kmin * (kmin - 1) / 2

      DO k = kmin,ny

C  -----  Ux = E setzen, wenn LUxE = .TRUE.
C	  w = (A)t * (Ux)i * x
	h = Zero
	DO j = 1,nx
	  IF ( LUxE ) Ux(j) = Eins
	  h = h + A(j,k) * x(j) / Ux(j)
	END DO
	w(k) = h

C  -----  Uyi = (Uy)i = (A)t * (Ux)i * A
	DO i = 1,k
	  h = Zero
	  DO j = 1,nx
	    h = h + A(j,i) * A(j,k) / Ux(j)
	  END DO
	  l = l+1
	  Uyi(l) = h
	END DO

      END DO

C  -----  Uyi umspeichern zum Invertieren
      l = 0
      DO k = 1,ny
      DO i = 1,k
	l = l+1
	Uy(i,k) = Uyi(l)
	Uy(k,i) = Uyi(l)
      END DO
      END DO

C  -----  Invertieren von (Uy)i (in Uy) ergibt Uy
      CALL DefInv ( ny,Uy,.TRUE.,Ld )
      IF (Ld .LT. ny) STOP
     -	 'Linear: Matrix (Uy)i nicht invertierbar. Fehlerstop 2'

C  -----  y = Uy * w
      DO i = 1,ny
	h = Zero
	DO k = 1,ny
	  h = h + Uy(i,k) * w(k)
	END DO
	y(i) = h
      END DO

C  -----  z = A * y
      DO i = 1,nx
	h = Zero
	DO k = 1,ny
	  h = h + A(i,k) * y(k)
	END DO
	z(i) = h
      END DO

C  -----  u0, t0, t1, t2
      chiq = Zero
      h0 = Zero
      h1 = Zero
      n2 = 0
      DO i = 1,nx
	h = x(i) - z(i)
	chiq = chiq + h * h / Ux(i)
	h0 = h0 + h * x(i)
	IF (i .GT. 1) THEN
	  h  = h * (x(i-1) - z(i-1))
	  h1 = h1 + h
	  IF (h .GT. Zero) n2 = n2 + 1
	  IF (h .LT. Zero) n2 = n2 - 1
	ENDIF
      END DO
      u0 = DSQRT( h0 / (nx - ny) )
      t0 = (chiq - nx + ny) / DSQRT( 2.0D0 * (nx - ny) )
      t1 = h1 / h0 * DSQRT( Eins * (nx - ny) )
      t2 = n2 / DSQRT( Eins * (nx - 1) )

      RETURN
      END
C --------------

      SUBROUTINE  DefInv ( n,S,Inv,d )

C   DefInv : Pruefen, ob eine symmetrische Matrix S positiv
C   ------   (semi)definit ist, und die Matrix S invertieren,
C	     wenn verlangt und wenn sie positiv definit ist
C							Weise 12.12.1993
C   Eingabe: n, S(n,n), Inv
C   Ausgabe:
C     Wenn S positiv (semi)definit ist: d = Rang(S) <= n, und
C	die obere Dreieckmatrix T mit (T)t*T = S. (T)t ersetzt S
C	(Cholesky-Zerlegung, (T)t transponierte Matrix T).
C     Wenn S positiv definit ist und Inv = .TRUE.: d = n,
C	und S wird durch ihre inverse Matrix (S)i ersetzt.
C     Sonst d = -1, und S ist ueberschrieben.

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER  ( Zero = 0.0D0, Eins = 1.0D0 )
      INTEGER	 d, h
      LOGICAL	 Inv
      DIMENSION  S(n,n)

      d = -1
      h =  0
      DO i = 1,n
      DO k = i,n
	p = S(i,k)
	IF (i .GT. 1) THEN
	  DO j = 1,i-1
	    p = p - S(i,j) * S(k,j)
	  END DO
	ENDIF
	IF (k .EQ. i) THEN
	  IF (p .LT. Zero) RETURN
	  IF (p .GT. Zero) THEN
	    S(i,i) = DSQRT( p )
	    h = h + 1
	  ENDIF
	 ELSE
	  IF (S(i,i) .NE. Zero) THEN
	    S(k,i) = p / S(i,i)
	   ELSE
	    IF (p .NE. Zero) RETURN
	  ENDIF
	ENDIF
      END DO
      END DO
      d = h
      IF (h.LT.n .OR. .NOT.Inv) RETURN

C   Invertieren der oberen Dreieckmatrix T ( (T)t liegt in S )
      DO k = 1,n
	S(k,k) = Eins / S(k,k)
	IF (k .NE. 1) THEN
	  DO i = k-1,1,-1
	    p = Zero
	    DO j = i+1,k
	      p = p + S(j,i) * S(j,k)
	    END DO
	    S(i,k) = -p * S(i,i)
	  END DO
	ENDIF
      END DO

C   (S)i = (T)i * ((T)i)t  ( (T)i liegt in S )
      DO i = n,1,-1
      DO k = 1,n
	IF (k .GT. i) THEN
	  S(i,k) = S(k,i)
	 ELSE
	  p = Zero
	  DO j = i,n
	    p = p + S(i,j) * S(k,j)
	  END DO
	  S(i,k) = p
	ENDIF
      END DO
      END DO

      RETURN
      END
C ==============
