
      PROGRAM  WIEDER

C   WIEDER : Auswertung bei mehrmaliger gemeinsamer	      23.11.1997
C   ------   indirekter Messung mehrerer Messgroessen		   Weise
C							 WIEDER.FOR 3.11
C
C   nx, ny		  Anzahl der Eingangs- bzw. Ergebnisgroessen
C   m	 (m > 1)	  Anzahl der gemeinsamen Einzelmessungen
C   X(nx), Y(ny)	  Eingangs- bzw. Ergebnisgroessen
C   V(nx,m)		  Messwerte von X
C   Ux(nx,nx), Uy(ny,ny)  Unsicherheitsmatrix von X bzw. Y
C   Rx(nx,nx), Ry(ny,ny)  Korrelationsmatrix von X bzw. Y
C   stux(nx),  stuy(ny)   Standardunsicherheiten von X bzw. Y
C   D(nx,ny)		  Empfindlichkeitsmatrix
C   Ya(ny), H(nx)	  Hilfsgroessen
C   a			  Skalenfaktor

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER  ( nx = 3, ny = 2, m = 5, a = 1.0D3 )
      EXTERNAL	 Leiter
      DIMENSION  X(nx), V(nx,m), Ux(nx,nx), Rx(nx,nx), stux(nx),
     -		 Y(ny), Ya(ny),  Uy(ny,ny), Ry(ny,ny), stuy(ny),
     -		 D(ny,nx), H(nx)

C  -----  Eingangsdaten
      DATA   V / 5.007, 19.663, 1.0456,
     -		 4.994, 19.639, 1.0438,
     -		 5.005, 19.640, 1.0468,
     -		 4.990, 19.685, 1.0428,
     -		 4.999, 19.678, 1.0433 /

C  -----  Eingangsdaten vorbereiten
      CALL  UnsX ( nx,m,V, X,Ux,Rx,stux )

C  -----  Messergebnisse und Fortpflanzung der
C	  Unsicherheiten berechnen
      CALL  UnsY ( nx,X,stux,Rx,ny,Leiter, Y,Ya,Uy,Ry,stuy,D,H)

C  -----  Aufgabe beschreiben und Messwerte ausgeben
      OPEN(6,FILE = 'WIEDER.AUS')
      WRITE(6,10) m
 10   FORMAT(
     -	' WIEDER : Auswertung bei mehrmaliger gemeinsamer'/
     -	' ------   indirekter Messung mehrerer Messgroessen'//
     -	' Ergebnisgroessen (interessierende Messgroessen der',
     -	' Messaufgabe)'/
     -	'    Y(1)  Wirkwiderstand und'/
     -	'    Y(2)  Blindwiderstand eines elektrischen Leiters'//
     -	' Eingangsgroessen (am Leiter gemessene Groessen)'/
     -	'    X(1)  Scheitelspannung einer angelegten sinusfoermigen'/
     -	'             Wechselspannung'/
     -	'    X(2)  Scheitelstromstaerke des hindurchfliessenden'/
     -	'             Wechselstroms'/
     -	'    X(3)  Phasenverschiebungswinkel zwischen Wechsel-'/
     -	'             spannung und -strom'//
     -	' Messwerte     Messung    X(1)     X(2)     X(3)'/
     -	' von ',I4,5X,'   Nr.      in V    in mA    in rad'/
     -	' Messungen    ',36(1H-))
      DO l = 1,m
	WRITE(6,11) l, (V(k,l), k = 1,nx)
 11	FORMAT(17X,I2,6X,F5.3,3X,F6.3,4X,F6.4)
      END DO
      WRITE(6,12)
 12   FORMAT(14X,36(1H-))

C  -----  Vollstaendige Messergebnisse ausgeben
      WRITE(6,13) X, stux, Rx, Y(1)*a, Y(2)*a,
     -		  stuy(1)*a, stuy(2)*a, Ry(1,2)
 13   FORMAT(
     -	' Mittelwerte            ',F7.4,2X,F7.4,3X,F7.5/
     -	' Standardunsicherheiten ',F7.4,2X,F7.4,3X,F7.5//
     -	' Korrelationsmatrix     ',3(F6.3,3X)/2(24X,3(F6.3,3X)/)/
     -	' Messergebnisse     ',
     -	' Y(1) =',F8.3,' Ohm    Y(2) =',F8.3,' Ohm'/21X,2(18(1H-),4X)/
     -	' Standardunsicherheiten      ',F6.3,' Ohm',12X,F6.3,' Ohm'/
     -	' Korrelationskoeffizient     ',F6.3)
      CLOSE(6)

      END
C --------------

      SUBROUTINE  Leiter ( X,Y )

C   Leiter : Modell der Auswertung
C   ------   Wirk- und Blindwiderstand eines elektrischen Leiters
C	     Eingabe: Eingangsgroessen X
C	     Ausgabe: Ergebnisgroessen Y
C
C   X(1)  Scheitelspannung einer an einen elektrischen Leiter
C	    angelegten sinusfoermigen Wechselspannung
C   X(2)  Scheitelstromstaerke des hindurchfliessenden Wechselstroms
C   X(3)  Phasenverschiebungswinkel zwischen Wechselspannung und -strom
C   Y(1)  Wirkwiderstand und
C   Y(2)  Blindwiderstand des Leiters

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION  X(3), Y(2)

      h = X(1) / X(2)
      Y(1) = h * DCOS( X(3) )
      Y(2) = h * DSIN( X(3) )

      RETURN
      END
C --------------

      SUBROUTINE  UnsX ( nx,m,V, X,Ux,Rx,stux )

C   UnsX : Berechnung der Schaetzwerte, Unsicherheitsmatrix,
C   ----   Korrelationsmatrix und Standardunsicherheiten der
C	   Eingangsgroessen bei mehrmaliger gemeinsamer Messung
C	   Eingabe:  nx, m, V ;  Ausgabe:  X, Ux, Rx, stux
C
C   nx, m  (m > 1)	  Anzahl der Eingangsgroessen bzw. Messungen
C   X(nx)		  Eingangsgroessen
C   V(nx,m)		  Messwerte von X
C   Ux(nx,nx), Rx(nx,nx)  Unsicherheits- bzw. Korrelationsmatrix von X
C   stux(nx)		  Standardunsicherheiten von X

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER  ( Zero = 0.0D0, Eins = 1.0D0 )
      DIMENSION  X(nx), V(nx,m), Ux(nx,nx), Rx(nx,nx), stux(nx)

C  -----  X
      DO k = 1,nx
	a = Zero
	DO j = 1,m
	  a = a + V(k,j)
	END DO
	X(k) = a / m
      END DO

C  -----  Ux
      DO k = 1,nx
      DO l = k,1,-1
	a = Zero
	DO j = 1,m
	  a = a + (V(k,j) - X(k)) * (V(l,j) - X(l))
	END DO
	a = a / m / (m - 1)
	Ux(k,l) = a
	Ux(l,k) = a

C  -----  Rx, stux
	IF (k .EQ. l) THEN
	  stux(k) = DSQRT( a )
	  Rx(k,k) = Eins
	 ELSE
	  Rx(k,l) = Zero
	  b = stux(k) * stux(l)
	  IF (b .NE. Zero) Rx(k,l) = a / b
	  Rx(l,k) = Rx(k,l)
	ENDIF
      END DO
      END DO

      RETURN
      END
C --------------

      SUBROUTINE  UnsY ( nx,X,stux,Rx,ny,MODELL, Y,Ya,Uy,Ry,stuy,D,H )

C   UnsY : Berechnung der Schaetzwerte, Unsicherheitsmatrix,
C   ----   Korrelationsmatrix und Standardunsicherheiten der
C	   Ergebnisgroessen sowie der Empfindlichkeitsmatrix
C	   Eingabe:  nx, X, stux, Rx, ny, MODELL
C	   Ausgabe:  Y, Uy, Ry, stuy, D
C
C   nx, ny		  Anzahl der Eingangs- bzw. Ergebnisgroessen
C   X(nx), Y(ny)	  Eingangs- bzw. Ergebnisgroessen
C   Uy(ny,ny)		  Unsicherheitsmatrix von Y
C   Rx(nx,nx), Ry(ny,ny)  Korrelationsmatrix von X bzw. Y
C   stux(nx),  stuy(ny)   Standardunsicherheiten von X bzw. Y
C   D(nx,ny)		  Empfindlichkeitsmatrix
C   Ya(ny), H(nx)	  Hilfsgroessen
C   MODELL(X,Y) 	  Subroutine des Modells der Auswertung

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER  ( Zero = 0.0D0, Eins = 1.0D0, Zwei = 2.0D0 )
      DIMENSION  X(nx), Rx(nx,nx), stux(nx), D(ny,nx), H(nx),
     -		 Y(ny), Ya(ny), Uy(ny,ny), Ry(ny,ny), stuy(ny)

C  -----  D
      DO k = 1,nx
	a = stux(k)
	X(k) = X(k) + a / Zwei
	CALL  MODELL ( X,Ya )
	X(k) = X(k) - a
	CALL  MODELL ( X,Y )
	X(k) = X(k) + a / Zwei
	DO i = 1,ny
	  D(i,k) = Ya(i) - Y(i)
	END DO
      END DO

C  -----  Y
      CALL  MODELL ( X,Y )

C  -----  Uy
      DO i = 1,ny
	DO k = 1,nx
	  a = Zero
	  DO j = 1,nx
	    a = a + D(i,j) * Rx(j,k)
	  END DO
	  H(k) = a
	END DO
	DO j = i,1,-1
	  a = Zero
	  DO k = 1,nx
	    a = a + H(k) * D(j,k)
	  END DO
	  Uy(i,j) = a
	  Uy(j,i) = a

C  -----  Ry, stuy
	  IF (i .EQ. j) THEN
	    stuy(i) = DSQRT( a )
	    Ry(i,i) = Eins
	   ELSE
	    Ry(i,j) = Zero
	    b = stuy(i) * stuy(j)
	    IF (b .NE. Zero) Ry(i,j) = a / b
	    Ry(j,i) = Ry(i,j)
	  ENDIF
	END DO
      END DO

      RETURN
      END
C ==============
